Exception: RubyLLM::ModelNotFoundError
- Inherits:
-
StandardError
- Object
- StandardError
- RubyLLM::ModelNotFoundError
- Defined in:
- lib/ruby_llm/error.rb
Overview
Raised when a requested model id is not in the model registry.
Raised when a requested model id is not in the model registry.