Class: RubyLLM::Protocols::InvokeModel

Inherits:
RubyLLM::Protocol show all
Defined in:
lib/ruby_llm/protocols/invoke_model.rb,
lib/ruby_llm/protocols/invoke_model/nova_embeddings.rb,
lib/ruby_llm/protocols/invoke_model/stability_images.rb,
lib/ruby_llm/protocols/invoke_model/cohere_embeddings.rb,
lib/ruby_llm/protocols/invoke_model/embedding_batches.rb,
lib/ruby_llm/protocols/invoke_model/titan_text_embeddings.rb,
lib/ruby_llm/protocols/invoke_model/titan_multimodal_embeddings.rb

Overview

The AWS InvokeModel API, where the envelope is shared but each model family owns its request and response body. Subclasses under Protocols::InvokeModel implement one family each. Requests are signed by the provider, which knows the credentials and the signing service.

Defined Under Namespace

Modules: EmbeddingBatches Classes: CohereEmbeddings, NovaEmbeddings, StabilityImages, TitanMultimodalEmbeddings, TitanTextEmbeddings

Instance Attribute Summary

Attributes inherited from RubyLLM::Protocol

#config, #connection, #model, #provider

Method Summary

Methods inherited from RubyLLM::Protocol

abstract, #animate_later, #apply_compaction, #apply_compaction_headers, #apply_end_user, #cache_content, #compact, #complete, #count_tokens, #delete_cache, #embed, #extend_cache, #find_cache, #initialize, #list_models, #moderate, #ocr, #paint, #parse_error, #parse_image_responses, #post_image, #post_video, #preprocess_message, #raise_transcription_streaming_unsupported, #refresh_video_job, #render, #render_embedding, #render_transcription_options, #render_video_extension_payload, #rerank, #server_tool_aliases, #speak, #stream_speech, #stream_speech_response, #stream_transcription, #supports_embedding_media?, #tokenize, #tool_approval_response, #transcribe, #video_extension_attachment, #video_extension_url, #video_request_url

Methods included from RubyLLM::Protocol::BinaryStreaming

#stream_binary

Methods included from RubyLLM::Protocol::Streaming

build_on_data_handler, build_stream_error_response, error_chunk?, failed_http_status, faraday_1?, handle_data, handle_error_chunk, handle_error_event, handle_failed_response, handle_json_error_chunk, handle_sse, handle_stream, json_error_payload?, parse_error_from_json, parse_streaming_error, process_stream_chunk, raise_stream_error, stream_events, stream_response, stream_state

Constructor Details

This class inherits a constructor from RubyLLM::Protocol