Class: RubyLLM::Providers::Deepgram
- Inherits:
-
RubyLLM::Provider
- Object
- RubyLLM::Provider
- RubyLLM::Providers::Deepgram
- Defined in:
- lib/ruby_llm/providers/deepgram.rb
Overview
Integrates Deepgram transcription and speech synthesis.
Constant Summary
Constants included from Support::Inspectable
Support::Inspectable::TRUNCATE_AT
Instance Attribute Summary
Attributes inherited from RubyLLM::Provider
Class Method Summary collapse
Instance Method Summary collapse
- #api_base ⇒ Object
- #headers ⇒ Object
-
#parse_error(response) ⇒ Object
Deepgram reports failures as an err_msg alongside an err_code, which the generic error parser does not know to look for.
Methods inherited from RubyLLM::Provider
#animate_later, #assume_models_exist?, assume_models_exist?, #batch_cost, #batch_cost_amounts, #batch_cost_multiplier, #batch_protocol_name, #batch_results, #batch_status, #batches?, #cache_content, #cancel_batch, #capabilities, capabilities, #compact, #complete, #configuration_requirements, configured?, #configured?, configured_providers, configured_remote_providers, #count_tokens, #create_batch, #delete_cache, display_name, #download_file, #embed, #extend_cache, #files?, #find_batch, #find_cache, #find_file, #find_research_job, #initialize, #list_file_uris, #list_models, local?, #local?, local_providers, #long_context_pricing?, model_registry_files, model_required?, models_dev_alias, models_dev_model_id, #moderate, #name, #ocr, #paint, #preprocess_message, protocol, #protocol_for, protocols, #protocols, providers, register, remote?, remote_providers, #render, #render_embedding, #rerank, #research_later, resolve, resolve!, resolve_registry_id, #retry_delay, #slug, #speak, #tokenize, #tool_approval_response, #transcribe, #upload_file
Methods included from Support::Inspectable
#full_inspect, #inspect, #pretty_print
Constructor Details
This class inherits a constructor from RubyLLM::Provider
Class Method Details
.configuration_options ⇒ Object
27 28 29 |
# File 'lib/ruby_llm/providers/deepgram.rb', line 27 def i[deepgram_api_key deepgram_api_base] end |
.configuration_requirements ⇒ Object
31 32 33 |
# File 'lib/ruby_llm/providers/deepgram.rb', line 31 def configuration_requirements i[deepgram_api_key] end |
Instance Method Details
#api_base ⇒ Object
9 10 11 |
# File 'lib/ruby_llm/providers/deepgram.rb', line 9 def api_base @config.deepgram_api_base || 'https://api.deepgram.com' end |
#headers ⇒ Object
13 14 15 |
# File 'lib/ruby_llm/providers/deepgram.rb', line 13 def headers { 'Authorization' => "Token #{@config.deepgram_api_key}" } end |
#parse_error(response) ⇒ Object
Deepgram reports failures as an err_msg alongside an err_code, which the generic error parser does not know to look for.
19 20 21 22 23 24 |
# File 'lib/ruby_llm/providers/deepgram.rb', line 19 def parse_error(response) body = parse_error_body(response) return super unless body.is_a?(Hash) body['err_msg'] || super end |