Class: RubyLLM::Models
- Inherits:
-
Object
- Object
- RubyLLM::Models
- Includes:
- Enumerable
- Defined in:
- lib/ruby_llm/models.rb
Overview
Registry of available AI models and their capabilities.
Constant Summary collapse
- MODELS_DEV_PROVIDER_MAP =
{ 'openai' => 'openai', 'anthropic' => 'anthropic', 'google' => 'gemini', 'google-vertex' => 'vertexai', 'amazon-bedrock' => 'bedrock', 'deepseek' => 'deepseek', 'mistral' => 'mistral', 'openrouter' => 'openrouter', 'perplexity' => 'perplexity' }.freeze
- PROVIDER_PREFERENCE =
%w[ openai anthropic gemini vertexai bedrock openrouter deepseek mistral perplexity xai azure ollama gpustack ].freeze
Class Method Summary collapse
-
.add_provider_metadata(models_dev_model, provider_model) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity.
- .blank_value?(value) ⇒ Boolean
-
.fetch_from_providers(remote_only: true) ⇒ Object
Backwards-compatible wrapper used by specs.
-
.fetch_models_dev_models(existing_models) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity.
-
.fetch_provider_models(remote_only: true) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity.
- .filter_models(models) ⇒ Object
- .find_models_dev_model(key, models_dev_by_key) ⇒ Object
- .index_by_key(models) ⇒ Object
- .instance ⇒ Object
- .load_existing_models ⇒ Object
- .load_models(file = RubyLLM.config.model_registry_file) ⇒ Object
- .log_models_dev_fetch(models_dev_fetch) ⇒ Object
- .log_provider_fetch(provider_fetch) ⇒ Object
- .merge_models(provider_models, models_dev_models) ⇒ Object
- .merge_with_existing(existing_models, provider_fetch, models_dev_fetch) ⇒ Object
- .method_missing(method) ⇒ Object
- .models_dev_capabilities(model_data, modalities) ⇒ Object
- .models_dev_metadata(model_data, provider_key) ⇒ Object
- .models_dev_model_to_info(model_data, provider_slug, provider_key) ⇒ Object
- .models_dev_pricing(cost) ⇒ Object
- .normalize_embedding_modalities(data) ⇒ Object
- .normalize_models_dev_knowledge(value) ⇒ Object
- .normalize_models_dev_modalities(modalities) ⇒ Object
- .read_from_json(file = RubyLLM.config.model_registry_file) ⇒ Object
- .refresh!(remote_only: false) ⇒ Object
-
.resolve(model_id, provider: nil, assume_exists: false, config: nil) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity.
- .respond_to_missing?(method, include_private = false) ⇒ Boolean
- .schema_file ⇒ Object
Instance Method Summary collapse
- #all ⇒ Object
- #audio_models ⇒ Object
- #by_family(family) ⇒ Object
- #by_provider(provider) ⇒ Object
- #chat_models ⇒ Object
- #each ⇒ Object
- #embedding_models ⇒ Object
- #find(model_id, provider = nil) ⇒ Object
- #image_models ⇒ Object
-
#initialize(models = nil) ⇒ Models
constructor
A new instance of Models.
- #load_from_json!(file = RubyLLM.config.model_registry_file) ⇒ Object
- #refresh!(remote_only: false) ⇒ Object
- #resolve(model_id, provider: nil, assume_exists: false, config: nil) ⇒ Object
- #save_to_json(file = RubyLLM.config.model_registry_file) ⇒ Object
Constructor Details
#initialize(models = nil) ⇒ Models
Returns a new instance of Models.
410 411 412 |
# File 'lib/ruby_llm/models.rb', line 410 def initialize(models = nil) @models = self.class.filter_models(models || self.class.load_models) end |
Class Method Details
.add_provider_metadata(models_dev_model, provider_model) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity
281 282 283 284 285 286 287 288 289 290 291 292 293 294 |
# File 'lib/ruby_llm/models.rb', line 281 def (models_dev_model, provider_model) # rubocop:disable Metrics/PerceivedComplexity data = models_dev_model.to_h data[:name] = provider_model.name if blank_value?(data[:name]) data[:family] = provider_model.family if blank_value?(data[:family]) data[:created_at] = provider_model.created_at if blank_value?(data[:created_at]) data[:context_window] = provider_model.context_window if blank_value?(data[:context_window]) data[:max_output_tokens] = provider_model.max_output_tokens if blank_value?(data[:max_output_tokens]) data[:modalities] = provider_model.modalities.to_h if blank_value?(data[:modalities]) data[:pricing] = provider_model.pricing.to_h if blank_value?(data[:pricing]) data[:metadata] = provider_model..merge(data[:metadata] || {}) data[:capabilities] = (models_dev_model.capabilities + provider_model.capabilities).uniq (data) Model::Info.new(data) end |
.blank_value?(value) ⇒ Boolean
305 306 307 308 309 310 311 312 313 314 315 316 |
# File 'lib/ruby_llm/models.rb', line 305 def blank_value?(value) return true if value.nil? return value.empty? if value.is_a?(String) || value.is_a?(Array) if value.is_a?(Hash) return true if value.empty? return value.values.all? { |nested| blank_value?(nested) } end false end |
.fetch_from_providers(remote_only: true) ⇒ Object
Backwards-compatible wrapper used by specs.
102 103 104 |
# File 'lib/ruby_llm/models.rb', line 102 def fetch_from_providers(remote_only: true) fetch_provider_models(remote_only: remote_only)[:models] end |
.fetch_models_dev_models(existing_models) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity
151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 |
# File 'lib/ruby_llm/models.rb', line 151 def fetch_models_dev_models(existing_models) # rubocop:disable Metrics/PerceivedComplexity RubyLLM.logger.info 'Fetching models from models.dev API...' connection = Connection.basic do |f| f.request :json f.response :json, parser_options: { symbolize_names: true } end response = connection.get 'https://models.dev/api.json' providers = response.body || {} models = providers.flat_map do |provider_key, provider_data| provider_slug = MODELS_DEV_PROVIDER_MAP[provider_key.to_s] next [] unless provider_slug (provider_data[:models] || {}).values.map do |model_data| Model::Info.new(models_dev_model_to_info(model_data, provider_slug, provider_key.to_s)) end end { models: models.reject { |model| model.provider.nil? || model.id.nil? }, fetched: true } rescue StandardError => e RubyLLM.logger.warn("Failed to fetch models.dev (#{e.class}: #{e.}). Keeping existing.") { models: existing_models.select { |model| model.[:source] == 'models.dev' }, fetched: false } end |
.fetch_provider_models(remote_only: true) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity
69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 |
# File 'lib/ruby_llm/models.rb', line 69 def fetch_provider_models(remote_only: true) # rubocop:disable Metrics/PerceivedComplexity config = RubyLLM.config provider_classes = remote_only ? Provider.remote_providers.values : Provider.providers.values configured_classes = if remote_only Provider.configured_remote_providers(config) else Provider.configured_providers(config) end configured = configured_classes.select { |klass| provider_classes.include?(klass) } result = { models: [], fetched_providers: [], configured_names: configured.map(&:name), failed: [] } provider_classes.each do |provider_class| next if remote_only && provider_class.local? next unless provider_class.configured?(config) begin result[:models].concat(provider_class.new(config).list_models) result[:fetched_providers] << provider_class.slug rescue StandardError => e result[:failed] << { name: provider_class.name, slug: provider_class.slug, error: e } end end result[:fetched_providers].uniq! result end |
.filter_models(models) ⇒ Object
239 240 241 242 243 |
# File 'lib/ruby_llm/models.rb', line 239 def filter_models(models) models.reject do |model| model.provider.to_s == 'vertexai' && model.id.to_s.include?('/') end end |
.find_models_dev_model(key, models_dev_by_key) ⇒ Object
245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 |
# File 'lib/ruby_llm/models.rb', line 245 def find_models_dev_model(key, models_dev_by_key) # Direct match return models_dev_by_key[key] if models_dev_by_key[key] provider, model_id = key.split(':', 2) if provider == 'bedrock' normalized_id = model_id.sub(/^[a-z]{2}\./, '') context_override = nil normalized_id = normalized_id.gsub(/:(\d+)k\b/) do context_override = Regexp.last_match(1).to_i * 1000 '' end bedrock_model = models_dev_by_key["bedrock:#{normalized_id}"] if bedrock_model data = bedrock_model.to_h.merge(id: model_id) data[:context_window] = context_override if context_override return Model::Info.new(data) end end # VertexAI uses same models as Gemini return unless provider == 'vertexai' gemini_model = models_dev_by_key["gemini:#{model_id}"] return unless gemini_model # Return Gemini's models.dev data but with VertexAI as provider Model::Info.new(gemini_model.to_h.merge(provider: 'vertexai')) end |
.index_by_key(models) ⇒ Object
275 276 277 278 279 |
# File 'lib/ruby_llm/models.rb', line 275 def index_by_key(models) models.to_h do |model| ["#{model.provider}:#{model.id}", model] end end |
.instance ⇒ Object
36 37 38 |
# File 'lib/ruby_llm/models.rb', line 36 def instance @instance ||= new end |
.load_existing_models ⇒ Object
178 179 180 181 182 |
# File 'lib/ruby_llm/models.rb', line 178 def load_existing_models existing_models = instance&.all existing_models = read_from_json if existing_models.nil? || existing_models.empty? existing_models end |
.load_models(file = RubyLLM.config.model_registry_file) ⇒ Object
44 45 46 |
# File 'lib/ruby_llm/models.rb', line 44 def load_models(file = RubyLLM.config.model_registry_file) read_from_json(file) end |
.log_models_dev_fetch(models_dev_fetch) ⇒ Object
194 195 196 197 198 |
# File 'lib/ruby_llm/models.rb', line 194 def log_models_dev_fetch(models_dev_fetch) return if models_dev_fetch[:fetched] RubyLLM.logger.warn('Using cached models.dev data due to fetch failure.') end |
.log_provider_fetch(provider_fetch) ⇒ Object
184 185 186 187 188 189 190 191 192 |
# File 'lib/ruby_llm/models.rb', line 184 def log_provider_fetch(provider_fetch) RubyLLM.logger.info "Fetching models from providers: #{provider_fetch[:configured_names].join(', ')}" provider_fetch[:failed].each do |failure| RubyLLM.logger.warn( "Failed to fetch #{failure[:name]} models (#{failure[:error].class}: #{failure[:error].}). " \ 'Keeping existing.' ) end end |
.merge_models(provider_models, models_dev_models) ⇒ Object
217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 |
# File 'lib/ruby_llm/models.rb', line 217 def merge_models(provider_models, models_dev_models) models_dev_by_key = index_by_key(models_dev_models) provider_by_key = index_by_key(provider_models) all_keys = models_dev_by_key.keys | provider_by_key.keys models = all_keys.map do |key| models_dev_model = find_models_dev_model(key, models_dev_by_key) provider_model = provider_by_key[key] if models_dev_model && provider_model (models_dev_model, provider_model) elsif models_dev_model models_dev_model else provider_model end end filter_models(models).sort_by { |m| [m.provider, m.id] } end |
.merge_with_existing(existing_models, provider_fetch, models_dev_fetch) ⇒ Object
200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 |
# File 'lib/ruby_llm/models.rb', line 200 def merge_with_existing(existing_models, provider_fetch, models_dev_fetch) existing_by_provider = existing_models.group_by(&:provider) preserved_models = existing_by_provider .except(*provider_fetch[:fetched_providers]) .values .flatten provider_models = provider_fetch[:models] + preserved_models models_dev_models = if models_dev_fetch[:fetched] models_dev_fetch[:models] else existing_models.select { |model| model.[:source] == 'models.dev' } end merge_models(provider_models, models_dev_models) end |
.method_missing(method) ⇒ Object
139 140 141 142 143 144 145 |
# File 'lib/ruby_llm/models.rb', line 139 def method_missing(method, ...) if instance.respond_to?(method) instance.send(method, ...) else super end end |
.models_dev_capabilities(model_data, modalities) ⇒ Object
344 345 346 347 348 349 350 351 |
# File 'lib/ruby_llm/models.rb', line 344 def models_dev_capabilities(model_data, modalities) capabilities = [] capabilities << 'function_calling' if model_data[:tool_call] capabilities << 'structured_output' if model_data[:structured_output] capabilities << 'reasoning' if model_data[:reasoning] capabilities << 'vision' if modalities[:input].intersect?(%w[image video pdf]) capabilities.uniq end |
.models_dev_metadata(model_data, provider_key) ⇒ Object
374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 |
# File 'lib/ruby_llm/models.rb', line 374 def (model_data, provider_key) = { source: 'models.dev', provider_id: provider_key, open_weights: model_data[:open_weights], attachment: model_data[:attachment], temperature: model_data[:temperature], last_updated: model_data[:last_updated], status: model_data[:status], interleaved: model_data[:interleaved], cost: model_data[:cost], limit: model_data[:limit], knowledge: model_data[:knowledge] } .compact end |
.models_dev_model_to_info(model_data, provider_slug, provider_key) ⇒ Object
318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 |
# File 'lib/ruby_llm/models.rb', line 318 def models_dev_model_to_info(model_data, provider_slug, provider_key) modalities = normalize_models_dev_modalities(model_data[:modalities]) capabilities = models_dev_capabilities(model_data, modalities) created_date = [model_data[:release_date], model_data[:last_updated]] .find { |value| !value.to_s.strip.empty? } data = { id: model_data[:id], name: model_data[:name] || model_data[:id], provider: provider_slug, family: model_data[:family], created_at: created_date ? "#{created_date} 00:00:00 UTC" : nil, context_window: model_data.dig(:limit, :context), max_output_tokens: model_data.dig(:limit, :output), knowledge_cutoff: normalize_models_dev_knowledge(model_data[:knowledge]), modalities: modalities, capabilities: capabilities, pricing: models_dev_pricing(model_data[:cost]), metadata: (model_data, provider_key) } (data) data end |
.models_dev_pricing(cost) ⇒ Object
353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 |
# File 'lib/ruby_llm/models.rb', line 353 def models_dev_pricing(cost) return {} unless cost text_standard = { input_per_million: cost[:input], output_per_million: cost[:output], cached_input_per_million: cost[:cache_read], reasoning_output_per_million: cost[:reasoning] }.compact audio_standard = { input_per_million: cost[:input_audio], output_per_million: cost[:output_audio] }.compact pricing = {} pricing[:text_tokens] = { standard: text_standard } if text_standard.any? pricing[:audio_tokens] = { standard: audio_standard } if audio_standard.any? pricing end |
.normalize_embedding_modalities(data) ⇒ Object
296 297 298 299 300 301 302 303 |
# File 'lib/ruby_llm/models.rb', line 296 def (data) return unless data[:id].to_s.include?('embedding') modalities = data[:modalities].to_h modalities[:input] = ['text'] if modalities[:input].nil? || modalities[:input].empty? modalities[:output] = ['embeddings'] data[:modalities] = modalities end |
.normalize_models_dev_knowledge(value) ⇒ Object
400 401 402 403 404 405 406 407 |
# File 'lib/ruby_llm/models.rb', line 400 def normalize_models_dev_knowledge(value) return if value.nil? return value if value.is_a?(Date) Date.parse(value.to_s) rescue ArgumentError nil end |
.normalize_models_dev_modalities(modalities) ⇒ Object
391 392 393 394 395 396 397 398 |
# File 'lib/ruby_llm/models.rb', line 391 def normalize_models_dev_modalities(modalities) normalized = { input: [], output: [] } return normalized unless modalities normalized[:input] = Array(modalities[:input]).compact normalized[:output] = Array(modalities[:output]).compact normalized end |
.read_from_json(file = RubyLLM.config.model_registry_file) ⇒ Object
48 49 50 51 52 53 54 |
# File 'lib/ruby_llm/models.rb', line 48 def read_from_json(file = RubyLLM.config.model_registry_file) data = File.exist?(file) ? File.read(file) : '[]' models = JSON.parse(data, symbolize_names: true).map { |model| Model::Info.new(model) } filter_models(models) rescue JSON::ParserError [] end |
.refresh!(remote_only: false) ⇒ Object
56 57 58 59 60 61 62 63 64 65 66 67 |
# File 'lib/ruby_llm/models.rb', line 56 def refresh!(remote_only: false) existing_models = load_existing_models provider_fetch = fetch_provider_models(remote_only: remote_only) log_provider_fetch(provider_fetch) models_dev_fetch = fetch_models_dev_models(existing_models) log_models_dev_fetch(models_dev_fetch) merged_models = merge_with_existing(existing_models, provider_fetch, models_dev_fetch) @instance = new(merged_models) end |
.resolve(model_id, provider: nil, assume_exists: false, config: nil) ⇒ Object
rubocop:disable Metrics/PerceivedComplexity
106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 |
# File 'lib/ruby_llm/models.rb', line 106 def resolve(model_id, provider: nil, assume_exists: false, config: nil) # rubocop:disable Metrics/PerceivedComplexity config ||= RubyLLM.config provider_class = provider ? Provider.providers[provider.to_sym] : nil if provider_class temp_instance = provider_class.new(config) assume_exists = true if temp_instance.local? || temp_instance.assume_models_exist? end if assume_exists raise ArgumentError, 'Provider must be specified if assume_exists is true' unless provider provider_class ||= raise(Error, "Unknown provider: #{provider.to_sym}") provider_instance = provider_class.new(config) model = if provider_instance.local? begin Models.find(model_id, provider) rescue ModelNotFoundError nil end end model ||= Model::Info.default(model_id, provider_instance.slug) else model = Models.find model_id, provider provider_class = Provider.providers[model.provider.to_sym] || raise(Error, "Unknown provider: #{model.provider}") provider_instance = provider_class.new(config) end [model, provider_instance] end |
.respond_to_missing?(method, include_private = false) ⇒ Boolean
147 148 149 |
# File 'lib/ruby_llm/models.rb', line 147 def respond_to_missing?(method, include_private = false) instance.respond_to?(method, include_private) || super end |
.schema_file ⇒ Object
40 41 42 |
# File 'lib/ruby_llm/models.rb', line 40 def schema_file File.('models_schema.json', __dir__) end |
Instance Method Details
#all ⇒ Object
422 423 424 |
# File 'lib/ruby_llm/models.rb', line 422 def all @models end |
#audio_models ⇒ Object
446 447 448 |
# File 'lib/ruby_llm/models.rb', line 446 def audio_models self.class.new(all.select { |m| m.type == 'audio' || m.modalities.output.include?('audio') }) end |
#by_family(family) ⇒ Object
454 455 456 |
# File 'lib/ruby_llm/models.rb', line 454 def by_family(family) self.class.new(all.select { |m| m.family == family.to_s }) end |
#by_provider(provider) ⇒ Object
458 459 460 |
# File 'lib/ruby_llm/models.rb', line 458 def by_provider(provider) self.class.new(all.select { |m| m.provider == provider.to_s }) end |
#chat_models ⇒ Object
438 439 440 |
# File 'lib/ruby_llm/models.rb', line 438 def chat_models self.class.new(all.select { |m| m.type == 'chat' }) end |
#each ⇒ Object
426 427 428 |
# File 'lib/ruby_llm/models.rb', line 426 def each(&) all.each(&) end |
#embedding_models ⇒ Object
442 443 444 |
# File 'lib/ruby_llm/models.rb', line 442 def self.class.new(all.select { |m| m.type == 'embedding' || m.modalities.output.include?('embeddings') }) end |
#find(model_id, provider = nil) ⇒ Object
430 431 432 433 434 435 436 |
# File 'lib/ruby_llm/models.rb', line 430 def find(model_id, provider = nil) if provider find_with_provider(model_id, provider) else find_without_provider(model_id) end end |
#image_models ⇒ Object
450 451 452 |
# File 'lib/ruby_llm/models.rb', line 450 def image_models self.class.new(all.select { |m| m.type == 'image' || m.modalities.output.include?('image') }) end |
#load_from_json!(file = RubyLLM.config.model_registry_file) ⇒ Object
414 415 416 |
# File 'lib/ruby_llm/models.rb', line 414 def load_from_json!(file = RubyLLM.config.model_registry_file) @models = self.class.read_from_json(file) end |
#refresh!(remote_only: false) ⇒ Object
462 463 464 |
# File 'lib/ruby_llm/models.rb', line 462 def refresh!(remote_only: false) self.class.refresh!(remote_only: remote_only) end |
#resolve(model_id, provider: nil, assume_exists: false, config: nil) ⇒ Object
466 467 468 |
# File 'lib/ruby_llm/models.rb', line 466 def resolve(model_id, provider: nil, assume_exists: false, config: nil) self.class.resolve(model_id, provider: provider, assume_exists: assume_exists, config: config) end |