Class: RubyLLM::Protocols::Converse
- Inherits:
-
RubyLLM::Protocol
- Object
- RubyLLM::Protocol
- RubyLLM::Protocols::Converse
- Includes:
- Converse::Chat, Converse::Media, Converse::Streaming
- Defined in:
- lib/ruby_llm/protocols/converse.rb,
lib/ruby_llm/protocols/converse/chat.rb,
lib/ruby_llm/protocols/converse/media.rb,
lib/ruby_llm/protocols/converse/batches.rb,
lib/ruby_llm/protocols/converse/streaming.rb,
lib/ruby_llm/protocols/converse/thinking_stream.rb
Overview
The AWS Bedrock Converse API. Requests are SigV4-signed by the provider.
Defined Under Namespace
Modules: Batches, Chat, Media, Streaming Classes: ThinkingStream
Constant Summary collapse
- REGION_PREFIXES =
%w[global us-gov us eu apac ap sa ca me af il au jp].freeze
- SERVER_TOOL_ALIASES =
Nova 2 models execute built-in tools server-side when the request names them as system tools.
{ web_search: lambda { || { tool: { systemTool: { name: 'nova_grounding' }.merge(Support::Utils.deep_symbolize_keys()) } } } }.freeze
Instance Attribute Summary
Attributes inherited from RubyLLM::Protocol
#config, #connection, #model, #provider
Instance Method Summary collapse
Methods inherited from RubyLLM::Protocol
abstract, #animate_later, #apply_compaction, #apply_compaction_headers, #apply_end_user, #cache_content, #compact, #complete, #count_tokens, #delete_cache, #embed, #extend_cache, #find_cache, #initialize, #list_models, #moderate, #ocr, #paint, #parse_error, #parse_image_responses, #post_image, #post_video, #preprocess_message, #raise_transcription_streaming_unsupported, #refresh_video_job, #render, #render_embedding, #render_transcription_options, #render_video_extension_payload, #rerank, #speak, #stream_speech, #stream_speech_response, #stream_transcription, #supports_embedding_media?, #tokenize, #tool_approval_response, #transcribe, #video_extension_attachment, #video_extension_url, #video_request_url
Methods included from RubyLLM::Protocol::BinaryStreaming
Methods included from RubyLLM::Protocol::Streaming
build_on_data_handler, build_stream_error_response, error_chunk?, failed_http_status, faraday_1?, handle_data, handle_error_chunk, handle_error_event, handle_failed_response, handle_json_error_chunk, handle_sse, handle_stream, json_error_payload?, parse_error_from_json, parse_streaming_error, process_stream_chunk, raise_stream_error, stream_events, stream_response, stream_state
Constructor Details
This class inherits a constructor from RubyLLM::Protocol
Instance Method Details
#server_tool_aliases ⇒ Object
21 22 23 |
# File 'lib/ruby_llm/protocols/converse.rb', line 21 def server_tool_aliases SERVER_TOOL_ALIASES end |