Class: RubyLLM::ResearchJob::Error
- Inherits:
-
Error
- Object
- Error
- RubyLLM::ResearchJob::Error
- Defined in:
- lib/ruby_llm/research_job.rb
Overview
A research failure that retains the job for inspection or recovery.
Direct Known Subclasses
Instance Attribute Summary collapse
-
#job ⇒ Object
readonly
The research job, including its remote ID and last known state.
Instance Method Summary collapse
-
#initialize(message, job:, response: nil) ⇒ Error
constructor
Creates an error retaining
joband the optional HTTP response.
Constructor Details
#initialize(message, job:, response: nil) ⇒ Error
Creates an error retaining job and the optional HTTP response.
24 25 26 27 |
# File 'lib/ruby_llm/research_job.rb', line 24 def initialize(, job:, response: nil) @job = job super(, response:) end |
Instance Attribute Details
#job ⇒ Object (readonly)
The research job, including its remote ID and last known state.
21 22 23 |
# File 'lib/ruby_llm/research_job.rb', line 21 def job @job end |