Exception: RubyLLM::UnsupportedAttachmentError

Inherits:
StandardError
  • Object
show all
Defined in:
lib/ruby_llm/error.rb

Overview

Raised when RubyLLM cannot format an attachment for the selected provider.

Constant Summary collapse

GUIDANCE =
'Consider using a model that supports this attachment type.'

Instance Method Summary collapse

Constructor Details

#initialize(type = nil) ⇒ UnsupportedAttachmentError

Returns a new instance of UnsupportedAttachmentError.



31
32
33
34
35
# File 'lib/ruby_llm/error.rb', line 31

def initialize(type = nil)
  message = 'Unsupported attachment type'
  message = "#{message}: #{type}" if type
  super("#{message}. #{GUIDANCE}")
end