Exception: RubyLLM::UnsupportedAttachmentError
- Inherits:
-
StandardError
- Object
- StandardError
- RubyLLM::UnsupportedAttachmentError
- Defined in:
- lib/ruby_llm/error.rb
Overview
Raised when RubyLLM cannot format an attachment for the selected provider.
Constant Summary collapse
- GUIDANCE =
'Consider using a model that supports this attachment type.'
Instance Method Summary collapse
-
#initialize(type = nil) ⇒ UnsupportedAttachmentError
constructor
A new instance of UnsupportedAttachmentError.
Constructor Details
#initialize(type = nil) ⇒ UnsupportedAttachmentError
Returns a new instance of UnsupportedAttachmentError.
31 32 33 34 35 |
# File 'lib/ruby_llm/error.rb', line 31 def initialize(type = nil) = 'Unsupported attachment type' = "#{}: #{type}" if type super("#{}. #{GUIDANCE}") end |