Exception: RubyLLM::UnsupportedAttachmentError
- Defined in:
- lib/ruby_llm/error.rb
Overview
Raised when an attachment cannot be formatted for the selected provider, for example an audio file sent to a model without audio input.
Constant Summary collapse
- GUIDANCE =
:nodoc:
'Consider using a model that supports this attachment type.'
Instance Attribute Summary
Attributes inherited from Error
Instance Method Summary collapse
-
#initialize(type = nil) ⇒ UnsupportedAttachmentError
constructor
:nodoc:.
Methods inherited from Error
Constructor Details
#initialize(type = nil) ⇒ UnsupportedAttachmentError
:nodoc:
96 97 98 99 100 |
# File 'lib/ruby_llm/error.rb', line 96 def initialize(type = nil) # :nodoc: = 'Unsupported attachment type' = "#{}: #{type}" if type super("#{}. #{GUIDANCE}") end |